@@ -3,19 +3,29 @@ import os
 import re
 import subprocess
 from itertools import chain
-from typing import Optional, List, Callable, Set, Container, Mapping, FrozenSet
+from typing import (
+    Optional,
+    List,
+    Callable,
+    Set,
+    Mapping,
+    FrozenSet,
+    Tuple,
+    Iterable,
+    Container,
+)
 
 from debian.deb822 import Deb822
 
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.commands.debputy_cmd.output import IOBasedOutputStyling
 from debputy.dh.debhelper_emulation import CannotEmulateExecutableDHConfigFile
-from debputy.dh_migration.migrators import MIGRATORS
+from debputy.dh.dh_assistant import read_dh_addon_sequences
+from debputy.dh_migration.migrators import Migrator, MigrationTarget
 from debputy.dh_migration.migrators_impl import (
     INTEGRATION_MODE_DH_DEBPUTY,
     INTEGRATION_MODE_DH_DEBPUTY_RRR,
 )
-from debputy.dh.dh_assistant import read_dh_addon_sequences
 from debputy.dh_migration.models import (
     FeatureMigration,
     AcceptableMigrationIssues,
@@ -29,15 +39,20 @@ from debputy.plugin.api import VirtualPath
 from debputy.plugin.api.spec import DebputyIntegrationMode, INTEGRATION_MODE_FULL
 from debputy.util import _error, _warn, _info, escape_shell, assume_not_none
 
-SUPPORTED_MIGRATIONS: Mapping[
-    DebputyIntegrationMode, FrozenSet[DebputyIntegrationMode]
-] = {
+SUPPORTED_MIGRATIONS: Mapping[MigrationTarget, FrozenSet[MigrationTarget]] = {
     INTEGRATION_MODE_FULL: frozenset([INTEGRATION_MODE_FULL]),
     INTEGRATION_MODE_DH_DEBPUTY: frozenset(
-        [INTEGRATION_MODE_DH_DEBPUTY, INTEGRATION_MODE_FULL]
+        [
+            "dh-single-to-multi-binary",
+            "dh-package-prefixed-config-files",
+            INTEGRATION_MODE_DH_DEBPUTY,
+            INTEGRATION_MODE_FULL,
+        ]
     ),
     INTEGRATION_MODE_DH_DEBPUTY_RRR: frozenset(
         [
+            "dh-single-to-multi-binary",
+            "dh-package-prefixed-config-files",
             INTEGRATION_MODE_DH_DEBPUTY_RRR,
             INTEGRATION_MODE_DH_DEBPUTY,
             INTEGRATION_MODE_FULL,
@@ -192,16 +207,16 @@ def _check_migration_target(
             and migration_target not in supported_migrations
         ):
             _error(
-                f"Cannot migrate from {detected_migration_target} to {migration_target}"
+                f"Cannot migrate apply migration {migration_target} as it conflicts with the current state (which is {detected_migration_target})"
             )
 
     if migration_target is not None:
         resolved_migration_target = migration_target
-        _info(f'Using "{resolved_migration_target}" as migration target as requested')
+        _info(f'Using "{resolved_migration_target}" as migration (requested)')
     else:
         if detected_migration_target is not None:
             _info(
-                f'Using "{detected_migration_target}" as migration target based on the packaging'
+                f'Using "{detected_migration_target}" as migration (based on the packaging)'
             )
         else:
             detected_migration_target = INTEGRATION_MODE_DH_DEBPUTY
@@ -213,13 +228,135 @@ def _check_migration_target(
     return resolved_migration_target
 
 
-def migrate_from_dh(
+def _read_git_status_lines(
+    lines: Iterable[bytes],
+) -> Iterable[Tuple[str, str, Optional[str]]]:
+    line_iter = iter(lines)
+    while True:
+        try:
+            line = next(line_iter)
+        except StopIteration:
+            return
+        if not line:
+            # We get a final empty line
+            continue
+        if len(line) < 4:
+            _error(
+                f"Internal error: Got out of sync with the `git status --porcelain=v1 -z` output ({len(line)=})"
+            )
+        status_marker = line[0:2].decode("utf-8")
+        if line[2] != 32:  # Space
+            _error(
+                "Internal error: Got out of sync with the `git status --porcelain=v1 -z` output (expected status + filename)"
+            )
+        if status_marker.startswith(("R", "C")) or status_marker.endswith(("R", "C")):
+            try:
+                second_filename = next(line_iter).decode("utf-8")
+            except StopIteration:
+                _error(
+                    "Internal error: Expected one more line of output from `git status` but it was not there"
+                )
+        else:
+            second_filename = None
+        filename = line[3:].decode("utf-8")
+        yield status_marker, filename, second_filename
+
+
+def _git_status() -> Tuple[bool, Container[str]]:
+    try:
+        top_level = (
+            subprocess.check_output(
+                ["git", "rev-parse", "--show-toplevel"],
+                stderr=subprocess.DEVNULL,
+                cwd="debian",
+            )
+            .strip()
+            .decode("utf-8")
+        )
+    except (subprocess.CalledProcessError, FileNotFoundError):
+        return False, frozenset()
+
+    if os.path.realpath(os.getcwd()) != os.path.realpath(top_level):
+        # Patches welcome. The primary problem is to have the status output match the `_check_vcs_clashes` method.
+        _error(
+            f"Unsupported git repo: The `debputy` command only supports `git` when cwd is the git root (git root is: {top_level}). Please use --ignore-vcs to continue."
+        )
+
+    try:
+        r = subprocess.check_output(
+            ["git", "status", "--porcelain=v1", "-z"],
+            stderr=subprocess.DEVNULL,
+            cwd="debian",
+        )
+    except subprocess.CalledProcessError:
+        _error(
+            "The `git status --porcelain=v1 -z` command returned non-zero (note: the command outputs binary). Please use `--ignore-vcs` to continue anyway."
+        )
+    except FileNotFoundError:
+        _error(
+            "Could not run `git status` and there is a `.git` directory. Please use `--ignore-vcs` to continue anyway."
+        )
+
+    untracked_files = set()
+    for status_marker, filename, second_filename in _read_git_status_lines(
+        r.split(b"\0")
+    ):
+        if status_marker not in ("??", "!!"):
+            _error(
+                "The current git tree is not clean, please commit or stash any pending changes or use `--ignore-vcs` to continue anyway."
+            )
+        # There is never a second file for `??` or `!!`
+        assert second_filename is None
+        untracked_files.add(filename)
+    return True, untracked_files
+
+
+def _check_vcs_clashes(
+    manifest: HighLevelManifest,
+    migrations: List[FeatureMigration],
+    untracked_files: Container[str],
+) -> None:
+    all_affected_files = []
+    if any(m.successful_manifest_changes for m in migrations):
+        all_affected_files.append(manifest.manifest_path)
+
+    for previous_path, new_path in (
+        p for m in migrations for p in m.rename_paths_on_success
+    ):
+        all_affected_files.append(previous_path)
+        all_affected_files.append(new_path)
+
+    all_affected_files.extend(p for m in migrations for p in m.remove_paths_on_success)
+
+    clashing_paths = [p for p in all_affected_files if p in untracked_files]
+
+    if clashing_paths:
+        print()
+        _warn(
+            "The following untracked or ignored paths would be affected by this migration"
+        )
+        _warn("")
+        for path in clashing_paths:
+            _warn(f" * {path}")
+        _warn("")
+        _warn("Since they are not tracked, the VCS cannot undo the changes to them")
+        _warn("if the migration continued")
+        print()
+        _error(
+            "Please stash, commit or remove the above files before continuing. Alternatively, use --ignore-vcs"
+        )
+
+
+def perform_migration(
     fo: IOBasedOutputStyling,
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     permit_destructive_changes: Optional[bool],
     migration_target: DebputyIntegrationMode,
     manifest_parser_factory: Callable[[str], HighLevelManifest],
+    migrators: List[Migrator],
+    *,
+    ignore_vcs: bool = False,
 ) -> None:
     migrations = []
     compat = _dh_compat_level()
@@ -231,8 +368,17 @@ def migrate_from_dh(
     debian_dir = manifest.debian_dir
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
 
+    if not ignore_vcs:
+        uses_git, untracked_files = _git_status()
+    else:
+        uses_git = False
+        untracked_files = frozenset()
+
+    if permit_destructive_changes is None and uses_git:
+        permit_destructive_changes = True
+
     try:
-        for migrator in MIGRATORS[migration_target]:
+        for migrator in migrators:
             feature_migration = FeatureMigration(migrator.__name__, fo)
             migrator(
                 debian_dir,
@@ -299,6 +445,9 @@ def migrate_from_dh(
         )
     )
 
+    if uses_git:
+        _check_vcs_clashes(manifest, migrations, untracked_files)
+
     _print_migration_summary(
         fo,
         migrations,
@@ -322,7 +471,7 @@ def migrate_from_dh(
             mutable_manifest.write_to(fd)
 
         try:
-            _info("Verifying the generating manifest")
+            _info("Verifying the new manifest")
             manifest_parser_factory(new_manifest_path)
         except ManifestParseException as e:
             raise AssertionError(
@@ -333,6 +482,8 @@ def migrate_from_dh(
             if os.path.isfile(manifest.manifest_path):
                 os.rename(manifest.manifest_path, manifest.manifest_path + ".orig")
             os.rename(new_manifest_path, manifest.manifest_path)
+            if uses_git:
+                subprocess.check_call(["git", "add", manifest.manifest_path])
             _info(f"Updated manifest {manifest.manifest_path}")
         else:
             _info(
@@ -350,18 +501,20 @@ def migrate_from_dh(
             _info("Paths being renamed:")
         else:
             _info("Migration *would* rename the following paths:")
+        prefix = "git " if uses_git else ""
         for previous_path, new_path in (
             p for m in migrations for p in m.rename_paths_on_success
         ):
-            _info(f"   mv {escape_shell(previous_path, new_path)}")
+            _info(f"   {prefix}mv {escape_shell(previous_path, new_path)}")
 
     if removals:
         if permit_destructive_changes:
             _info("Removals:")
         else:
             _info("Migration *would* remove the following files:")
+        cmd = "git rm" if uses_git else "rm -f"
         for path in (p for m in migrations for p in m.remove_paths_on_success):
-            _info(f"  rm -f {escape_shell(path)}")
+            _info(f"  {cmd} {escape_shell(path)}")
 
     if permit_destructive_changes is None:
         print()
@@ -369,12 +522,7 @@ def migrate_from_dh(
             "If you would like to perform the migration, please re-run with --apply-changes."
         )
     elif permit_destructive_changes:
-        for previous_path, new_path in (
-            p for m in migrations for p in m.rename_paths_on_success
-        ):
-            os.rename(previous_path, new_path)
-        for path in (p for m in migrations for p in m.remove_paths_on_success):
-            os.unlink(path)
+        _apply_renames_and_removals(migrations, uses_git)
 
         print()
         _info("Migrations performed successfully")
@@ -382,6 +530,32 @@ def migrate_from_dh(
         _info(
             "Remember to validate the resulting binary packages after rebuilding with debputy"
         )
+        if uses_git:
+            _info(
+                "Use `git commit` to commit the changes or use `git reset` / `git restore` to undo them."
+            )
     else:
         print()
         _info("No migrations performed as requested")
+
+
+def _apply_renames_and_removals(
+    migrations: List[FeatureMigration],
+    uses_git: bool,
+) -> None:
+    for previous_path, new_path in (
+        p for m in migrations for p in m.rename_paths_on_success
+    ):
+        if uses_git:
+            subprocess.check_call(["git", "mv", previous_path, new_path])
+        else:
+            os.rename(previous_path, new_path)
+
+    files_being_removed = [p for m in migrations for p in m.remove_paths_on_success]
+    if uses_git and files_being_removed:
+        command = ["git", "rm"]
+        command.extend(files_being_removed)
+        subprocess.check_call(command)
+    elif files_being_removed:
+        for path in (p for m in migrations for p in m.remove_paths_on_success):
+            os.unlink(path)
