@@ -40,10 +40,24 @@ from debputy.util import (
 )
 from debputy.util import _warn, _info
 from ._deb_options_profiles import DebBuildOptionsAndProfiles
+from ._manifest_constants import (
+    MK_CONFFILE_MANAGEMENT,
+    MK_INSTALLATIONS,
+    MK_PACKAGES,
+    MK_INSTALLATION_SEARCH_DIRS,
+    MK_TRANSFORMATIONS,
+    MK_BINARY_VERSION,
+    MK_SERVICES,
+    MK_MANIFEST_REMOVE_DURING_CLEAN,
+)
 from .architecture_support import DpkgArchitectureBuildProcessValuesTable
 from .filesystem_scan import FSROOverlay
 from .installations import InstallRule, PPFInstallRule
-from .manifest_parser.base_types import BuildEnvironments, BuildEnvironmentDefinition
+from .manifest_parser.base_types import (
+    BuildEnvironments,
+    BuildEnvironmentDefinition,
+    FileSystemMatchRule,
+)
 from .manifest_parser.exceptions import ManifestParseException
 from .manifest_parser.parser_data import ParserContextData
 from .manifest_parser.util import AttributePath
@@ -156,6 +170,7 @@ class HighLevelManifestParser(ParserContextData):
         self._all_package_states = None
 
         self._install_rules: Optional[List[InstallRule]] = None
+        self._remove_during_clean_rules: List[FileSystemMatchRule] = []
         self._ownership_caches_loaded = False
         self._used = False
 
@@ -267,6 +282,7 @@ class HighLevelManifestParser(ParserContextData):
         return HighLevelManifest(
             self.manifest_path,
             self._mutable_yaml_manifest,
+            self._remove_during_clean_rules,
             self._install_rules,
             self._source_package,
             self.binary_packages,
@@ -553,10 +569,13 @@ class YAMLManifestParser(HighLevelManifestParser):
             "Mapping[str, PackageContextData[Mapping[str, Any]]]",
             parsed_data.get("packages", {}),
         )
-        install_rules = parsed_data.get("installations")
+        self._remove_during_clean_rules = parsed_data.get(
+            MK_MANIFEST_REMOVE_DURING_CLEAN, []
+        )
+        install_rules = parsed_data.get(MK_INSTALLATIONS)
         if install_rules:
             self._install_rules = install_rules
-        packages_parent_path = attribute_path["packages"]
+        packages_parent_path = attribute_path[MK_PACKAGES]
         for package_name_raw, pcd in packages_dict.items():
             definition_source = packages_parent_path[package_name_raw]
             package_name = pcd.resolved_package_name
@@ -570,20 +589,20 @@ class YAMLManifestParser(HighLevelManifestParser):
                         f'Cannot define rules for package "{package_name}" (at {definition_source.path}). It is an'
                         " auto-generated package."
                     )
-                binary_version = parsed.get("binary-version")
+                binary_version = parsed.get(MK_BINARY_VERSION)
                 if binary_version is not None:
                     package_state.binary_version = (
                         package_state.substitution.substitute(
                             binary_version,
-                            definition_source["binary-version"].path,
+                            definition_source[MK_BINARY_VERSION].path,
                         )
                     )
-                search_dirs = parsed.get("installation_search_dirs")
+                search_dirs = parsed.get(MK_INSTALLATION_SEARCH_DIRS)
                 if search_dirs is not None:
                     package_state.search_dirs = search_dirs
-                transformations = parsed.get("transformations")
-                conffile_management = parsed.get("conffile_management")
-                service_rules = parsed.get("services")
+                transformations = parsed.get(MK_TRANSFORMATIONS)
+                conffile_management = parsed.get(MK_CONFFILE_MANAGEMENT)
+                service_rules = parsed.get(MK_SERVICES)
                 if transformations:
                     package_state.transformations.extend(transformations)
                 if conffile_management:
